package cn.com.qmth.archive.service.impl;

import java.io.File;
import java.io.FileInputStream;
import java.io.FileNotFoundException;
import java.io.IOException;
import java.io.InputStream;
import java.time.Instant;
import java.time.LocalDateTime;
import java.time.ZoneId;
import java.util.ArrayList;
import java.util.Arrays;
import java.util.Date;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

import org.apache.commons.collections4.CollectionUtils;
import org.apache.commons.io.FileUtils;
import org.apache.commons.lang3.StringUtils;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import com.qmth.boot.core.exception.StatusException;
import com.qmth.boot.tools.excel.ExcelReader;
import com.qmth.boot.tools.excel.enums.ExcelType;
import com.qmth.boot.tools.excel.model.DataMap;

import cn.com.qmth.archive.config.SysProperty;
import cn.com.qmth.archive.dao.MarkingDao;
import cn.com.qmth.archive.service.MarkingService;
import cn.com.qmth.archive.utils.DateUtil;
import cn.com.qmth.archive.utils.FileUtil;

@Service
public class MarkingServiceImpl implements MarkingService {

    private final static Logger log = LoggerFactory.getLogger(MarkingService.class);

    private static final String[] EXCEL_HEADER = new String[] { "表名", "字段名" };

    @Autowired
    private SysProperty sysProperty;

    @Autowired
    private MarkingDao markingDao;

    @Override
    public void exportSql() {
        File dir = new File("./sql");
        if (dir.exists()) {
            FileUtil.deleteDirectory(dir);
        }
        dir.mkdirs();
        String workidStr = sysProperty.getWorkids();
        if (StringUtils.isBlank(workidStr)) {
            throw new StatusException("配置文件中workids为空");
        }
        List<Long> workids = new ArrayList<>();
        for (String wid : workidStr.split(",")) {
            if (StringUtils.isBlank(wid)) {
                throw new StatusException("配置文件中workids错误");
            }
            try {
                workids.add(Long.valueOf(wid));
            } catch (NumberFormatException e) {
                throw new StatusException("配置文件中workids错误");
            }
        }

        log.warn("*********************开始处理，workid：" + StringUtils.join(workids, ","));
        try {
            Map<String, List<String>> cols = getCols();
            getWork(cols, dir, workids);
            getWorkExam(cols, dir, workids);
            getStudentPaper(cols, dir, workids);
            getMarkRange(cols, dir, workids);
            getMarkResult(cols, dir, workids);
            getMarkTask(cols, dir, workids);
            getMarkTaskLimit(cols, dir, workids);
            getMarkItem(cols, dir, workids);
            getResultItem(cols, dir, workids);
            getRandomResultItem(cols, dir, workids);
            getScoreStep(cols, dir, workids);
            getRejectResult(cols, dir, workids);
            log.warn("*********************处理结束，成功");
        } catch (Exception e) {
            throw new StatusException("*********************处理结束，失败", e);
        }
    }

    private void getWork(Map<String, List<String>> cols, File dir, List<Long> workids) throws IOException {
        String tableName = "ec_m_mark_work";
        log.warn("*********************处理" + tableName);
        List<String> col = getCol(cols, tableName);
        List<Map<String, Object>> data = markingDao.getWork(col, workids);
        if (CollectionUtils.isEmpty(data)) {
            throw new StatusException(tableName + "表没有数据");
        }
        getSql(dir, col, tableName, data);
    }

    private void getWorkExam(Map<String, List<String>> cols, File dir, List<Long> workids) throws IOException {
        String tableName = "ec_m_mark_work_exam";
        log.warn("*********************处理" + tableName);
        List<String> col = getCol(cols, tableName);
        List<Map<String, Object>> data = markingDao.getWorkExam(col, workids);

        getSql(dir, col, tableName, data);
    }

    private void getStudentPaper(Map<String, List<String>> cols, File dir, List<Long> workids) throws IOException {
        String tableName = "ec_m_student_paper";
        log.warn("*********************处理" + tableName);
        List<String> col = getCol(cols, tableName);
        List<Map<String, Object>> data = markingDao.getStudentPaper(col, workids);
        getSql(dir, col, tableName, data);
    }

    private void getMarkRange(Map<String, List<String>> cols, File dir, List<Long> workids) throws IOException {
        String tableName = "ec_m_mark_range";
        log.warn("*********************处理" + tableName);
        List<String> col = getCol(cols, tableName);
        List<Map<String, Object>> data = markingDao.getMarkRange(col, workids);
        getSql(dir, col, tableName, data);
    }

    private void getMarkResult(Map<String, List<String>> cols, File dir, List<Long> workids) throws IOException {
        String tableName = "ec_m_mark_result";
        log.warn("*********************处理" + tableName);
        List<String> col = getCol(cols, tableName);
        List<Map<String, Object>> data = markingDao.getMarkResult(col, workids);
        getSql(dir, col, tableName, data);
    }

    private void getMarkTask(Map<String, List<String>> cols, File dir, List<Long> workids) throws IOException {
        String tableName = "ec_m_mark_task";
        log.warn("*********************处理" + tableName);
        List<String> col = getCol(cols, tableName);
        List<Map<String, Object>> data = markingDao.getMarkTask(col, workids);
        getSql(dir, col, tableName, data);
    }

    private void getMarkTaskLimit(Map<String, List<String>> cols, File dir, List<Long> workids) throws IOException {
        String tableName = "ec_m_mark_task_limit";
        log.warn("*********************处理" + tableName);
        List<String> col = getCol(cols, tableName);
        List<Map<String, Object>> data = markingDao.getMarkTaskLimit(col, workids);
        getSql(dir, col, tableName, data);
    }

    private void getMarkItem(Map<String, List<String>> cols, File dir, List<Long> workids) throws IOException {
        String tableName = "ec_m_mark_item";
        log.warn("*********************处理" + tableName);
        List<String> col = getCol(cols, tableName);
        List<Map<String, Object>> data = markingDao.getMarkItem(col, workids);
        getSql(dir, col, tableName, data);
    }

    private void getResultItem(Map<String, List<String>> cols, File dir, List<Long> workids) throws IOException {
        String tableName = "ec_m_result_item";
        log.warn("*********************处理" + tableName);
        List<String> col = getCol(cols, tableName);
        List<Map<String, Object>> data = markingDao.getResultItem(col, workids);
        getSql(dir, col, tableName, data);
    }

    private void getRandomResultItem(Map<String, List<String>> cols, File dir, List<Long> workids) throws IOException {
        String tableName = "ec_m_random_result_item";
        log.warn("*********************处理" + tableName);
        List<String> col = getCol(cols, tableName);
        List<Map<String, Object>> data = markingDao.getRandomResultItem(col, workids);
        getSql(dir, col, tableName, data);
    }

    private void getScoreStep(Map<String, List<String>> cols, File dir, List<Long> workids) throws IOException {
        String tableName = "ec_m_mark_score_step";
        log.warn("*********************处理" + tableName);
        List<String> col = getCol(cols, tableName);
        List<Map<String, Object>> data = markingDao.getScoreStep(col, workids);
        getSql(dir, col, tableName, data);
    }

    private void getRejectResult(Map<String, List<String>> cols, File dir, List<Long> workids) throws IOException {
        String tableName = "ec_m_reject_result";
        log.warn("*********************处理" + tableName);
        List<String> col = getCol(cols, tableName);
        List<Map<String, Object>> data = markingDao.getRejectResult(col, workids);
        getSql(dir, col, tableName, data);
    }

    private String trimAndNullIfBlank(String s) {
        if (StringUtils.isBlank(s)) {
            return null;
        }
        return s.trim();
    }

    private List<String> getCol(Map<String, List<String>> cols, String tableName) {
        List<String> col = cols.get(tableName);
        if (CollectionUtils.isEmpty(col)) {
            throw new StatusException(tableName + "表没有字段信息");
        }
        return col;
    }

    @SuppressWarnings("deprecation")
    private Map<String, List<String>> getCols() throws FileNotFoundException {
        File file = new File("./columns/columns.xlsx");
        if (!file.exists()) {
            throw new StatusException("columns下没有表字段文件");
        }
        InputStream inputStream = null;
        try {
            inputStream = new FileInputStream(file);
            List<DataMap> lineList = null;
            ExcelReader reader = ExcelReader.create(ExcelType.XLSX, inputStream, 0);
            try {
                lineList = reader.getDataMapList();
            } catch (Exception e) {
                throw new StatusException("Excel 解析失败");
            }
            if (!Arrays.equals(EXCEL_HEADER, reader.getColumnNames())) {
                throw new StatusException("Excel表头错误");
            }
            if (CollectionUtils.isEmpty(lineList)) {
                throw new StatusException("Excel无内容");
            }
            Map<String, List<String>> cols = new HashMap<>();
            for (int i = 1; i < lineList.size(); i++) {
                DataMap line = lineList.get(i);
                String tname = trimAndNullIfBlank(line.get(EXCEL_HEADER[0]));
                if (StringUtils.isBlank(tname)) {
                    throw new StatusException("第" + (i + 1) + "行表名为空");
                }
                String cname = trimAndNullIfBlank(line.get(EXCEL_HEADER[1]));
                if (StringUtils.isBlank(cname)) {
                    throw new StatusException("第" + (i + 1) + "行字段名为空");
                }
                List<String> tem = cols.get(tname);
                if (tem == null) {
                    tem = new ArrayList<>();
                    cols.put(tname, tem);
                }
                tem.add(cname);
            }
            return cols;
        } finally {
            if (inputStream != null) {
                try {
                    inputStream.close();
                } catch (IOException e) {
                }
            }
        }

    }

    private void getSql(File dir, List<String> col, String tableName, List<Map<String, Object>> data)
            throws IOException {
        File sql = new File(dir.getAbsolutePath() + "/" + tableName + ".sql");
        sql.createNewFile();
        if (CollectionUtils.isEmpty(data)) {
            return;
        }
        List<String> ret = new ArrayList<>();
        for (Map<String, Object> map : data) {
            ret.add(getLine(tableName, col, map));
        }
        FileUtils.writeLines(sql, "utf-8", ret, false);
    }

    private String getLine(String tbName, List<String> cols, Map<String, Object> line) {
        StringBuilder sb = new StringBuilder("INSERT INTO `" + tbName + "` (`");
        sb.append(StringUtils.join(cols, "`,`") + "`) ");
        sb.append("values (");
        for (String col : cols) {
            Object ob = line.get(col);
            if (ob == null) {
                sb.append("null,");
            } else {
                if (ob instanceof Boolean) {
                    Boolean tem = (Boolean) ob;
                    if (tem) {
                        sb.append("1,");
                    } else {
                        sb.append("0,");
                    }
                } else if (ob instanceof LocalDateTime) {
                    LocalDateTime tem = (LocalDateTime) ob;
                    Instant instant = tem.atZone(ZoneId.systemDefault()).toInstant();

                    // 将 Instant 转换为 Date
                    Date date = Date.from(instant);
                    sb.append("'" + DateUtil.format(date, DateUtil.DatePatterns.CHINA_DEFAULT) + "',");
                } else if (ob instanceof String) {
                    sb.append("'" + ob.toString() + "',");
                } else {
                    sb.append(ob.toString() + ",");
                }
            }
        }
        sb.deleteCharAt(sb.length() - 1);
        sb.append("); ");
        return sb.toString();
    }

}